import java.util.Arrays;

/**
 * Simple driver class that illustrates how the provided sort classes and
 * generators may be used.
 * 
 * @author Nathan Sprague
 * @version 10/2016
 *
 */
public class DemoDriver {

  /**
   * Return the total number of nanoseconds required for the provided sorting
   * algorithm to sort the provided array. A copy of the array is sorted. The
   * original will be unmodified.
   */
  public static long timeSort(Sorter<Integer> sorter, Integer[] items) {
    Integer[] copy = Arrays.copyOf(items, items.length);
    long start = System.nanoTime();
    sorter.sort(copy);
    return System.nanoTime() - start;
  }


  /**
   * Quicksort 100000 items and display the time in seconds.
   * 
   * @param args Ignored
   */
  public static void main(String[] args) {

    Sorter<Integer> sorter = new SelectionSorter<>();
    Generator gen = new RandomGenerator();
    Integer[] items = new Integer[3];
    items[0] = new Integer(20);
    items[1] = new Integer(30);
    items[2] = new Integer(3);
    

    long timeInNanoSeconds = timeSort(sorter, items);
    double timeInSeconds = timeInNanoSeconds / 1000000000.0;
    System.out.format("Time to quicksort %d items: %.8fs.", items.length, timeInSeconds);
  }

}
