/*
 * OpenDSA Project Distributed under the MIT License
 * 
 * Copyright (c) 2011-2016 - Ville Karavirta and Cliff Shaffer
 * 
 * Modifications by Nathan Sprague 2016.
 */

/**
 * Quicksort class.
 */
public class QuickSorter<T extends Comparable<T>> extends Sorter<T> {

  /**
   * Quicksort the provided array.
   */
  @Override
  public void sort(T[] items) {
    quicksort(items, 0, items.length - 1);
  }

  /**
   * Recursive helper method for quicksort.
   * 
   * @param items The items to sort
   * @param left The starting index of the region to sort
   * @param right The ending index of the region to sort.
   */
  private void quicksort(T[] items, int left, int right) {
    int pivotindex = findpivot(items, left, right);

    // curr will be the final position of the pivot item.
    int curr = partition(items, left, right, pivotindex);

    if ((curr - left) > 1) {
      quicksort(items, left, curr - 1); // Sort left partition
    }
    if ((right - curr) > 1) {
      quicksort(items, curr + 1, right); // Sort right partition
    }
  }

  /**
   * Helper method to select the pivot point.
   */
  private int findpivot(T[] items, int left, int right) {
    return (left + right) / 2;
  }

  /**
   * Partition the indicated region of the array. The pivot item will be placed
   * in its final sorted position, with all smaller elements moved to the left
   * and all larger elements moved to the right.
   * 
   * @return The final index of the pivot item.
   */
  private int partition(T[] items, int left, int right, int pivotindex) {

    T pivot = items[pivotindex];
    swap(items, pivotindex, right); // Stick the pivot at end
    pivotindex = right; // remember where it is.
    right--;


    while (left <= right) { // Move bounds inward until they meet
      while (items[left].compareTo(pivot) < 0) {
        left++;
      }
      while ((right >= left) && (items[right].compareTo(pivot) >= 0)) {
        right--;
      }
      if (right > left) {
        swap(items, left, right); // Swap out-of-place values
      }
    }
    swap(items, left, pivotindex); // Put pivot in place

    return left; // Return first position in right partition
  }

  public String toString() {
    return "Quicksort";
  }

}
